@@ -16,15 +16,30 @@ sub r {
     if ($INC{"mod_perl.pm"}) {
         return Apache->request;
     } elsif ($INC{"mod_perl2.pm"}) {
-        return Apache2->request;
+        return Apache2::RequestUtil->request;
     } else {
         die "Not running on mod_perl environment.";
     }
 }
 
-sub get_from_cache    { $_[0]->r->pnotes($_[1]) }
-sub add_to_cache      { $_[0]->r->pnotes($_[1], $_[2]) }
-sub update_cache      { $_[0]->r->pnotes($_[1], $_[2]) }
-sub remove_from_cache { delete $_[0]->r->pnotes->{$_[1]} }
+sub get_from_cache {
+    my $r = $_[0]->r or return;
+    $r->pnotes($_[1]);
+}
+
+sub add_to_cache {
+    my $r = $_[0]->r or return;
+    $r->pnotes($_[1], $_[2]);
+}
+
+sub update_cache {
+    my $r = $_[0]->r or return;
+    $r->pnotes($_[1], $_[2]);
+}
+
+sub remove_from_cache {
+    my $r = $_[0]->r or return;
+    delete $r->pnotes->{$_[1]};
+}
 
 1;
